@@ -1,7 +1,7 @@
 /* Android activity for Gforth on Android
 
   Authors: Bernd Paysan, Anton Ertl
-  Copyright (C) 2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Free Software Foundation, Inc.
+  Copyright (C) 2013,2014,2015,2016,2017,2018,2019,2020,2021,2022,2023 Free Software Foundation, Inc.
 
   This file is part of Gforth.
 
@@ -45,6 +45,7 @@ import android.hardware.Sensor;
 import android.hardware.SensorEvent;
 import android.hardware.SensorEventListener;
 import android.hardware.SensorManager;
+import android.hardware.usb.UsbManager;
 import android.content.Context;
 import android.view.View;
 import android.view.Window;
@@ -115,11 +116,12 @@ public class Gforth
     private PendingIntent pintent, gforthintent;
     private PowerManager powerManager;
     private NotificationManager notificationManager;
+    private UsbManager usbManager;
     private NotificationChannel notificationChannel;
     private WakeLock wl, wl_cpu;
     private GforthView mView;
     private InputStream gforthfd;
-
+    private Intent intent;
     private boolean started=false;
     private boolean libloaded=false;
     private boolean surfaced=false;
@@ -327,13 +329,17 @@ public class Gforth
 	@Override
 	public InputConnection onCreateInputConnection (EditorInfo outAttrs) {
 	    moutAttrs=outAttrs;
-	    outAttrs.inputType = (InputType.TYPE_CLASS_TEXT | /*
-				  InputType.TYPE_TEXT_FLAG_AUTO_COMPLETE | */
-				  InputType.TYPE_TEXT_FLAG_AUTO_CORRECT);
+	    outAttrs.inputType = (InputType.TYPE_CLASS_TEXT |
+				  InputType.TYPE_TEXT_VARIATION_LONG_MESSAGE |
+				  InputType.TYPE_TEXT_VARIATION_WEB_EDIT_TEXT |
+				  InputType.TYPE_TEXT_FLAG_AUTO_CORRECT |
+				  InputType.TYPE_TEXT_FLAG_MULTI_LINE);
 	    outAttrs.initialSelStart = mcurpos;
 	    outAttrs.initialSelEnd = mcurpos+mlen;
 	    outAttrs.packageName = "gnu.gforth";
-	    outAttrs.imeOptions = (EditorInfo.IME_FLAG_NO_FULLSCREEN);
+	    outAttrs.imeOptions = (EditorInfo.IME_FLAG_NO_FULLSCREEN |
+				   EditorInfo.IME_FLAG_NAVIGATE_NEXT |
+				   EditorInfo.IME_FLAG_NAVIGATE_PREVIOUS);
 	    mInputConnection = new MyInputConnection(this, true);
 	    return mInputConnection;
 	}
@@ -478,6 +484,8 @@ public class Gforth
     protected void onCreate(Bundle savedInstanceState) {
         ActivityInfo ai;
         String libname = "gforth";
+	intent = getIntent();
+	Log.v(TAG, "onCreate, intent.action="+intent.getAction()+" intent.data="+intent.getDataString());
 
 	gforth=this;
 	
@@ -495,7 +503,7 @@ public class Gforth
         mView.getViewTreeObserver().addOnGlobalLayoutListener(this);
 
 	try {
-            ai = getPackageManager().getActivityInfo(getIntent().getComponent(), PackageManager.GET_META_DATA);
+            ai = getPackageManager().getActivityInfo(intent.getComponent(), PackageManager.GET_META_DATA);
             if (ai.metaData != null) {
                 String ln = ai.metaData.getString(META_DATA_LIB_NAME);
                 if (ln != null) libname = ln;
@@ -523,6 +531,7 @@ public class Gforth
 	inputMethodManager=(InputMethodManager)getSystemService(Context.INPUT_METHOD_SERVICE);
 	powerManager=(PowerManager)getSystemService(Context.POWER_SERVICE);
 	notificationManager=(NotificationManager)getSystemService(Context.NOTIFICATION_SERVICE);
+	usbManager=(UsbManager) getSystemService(Context.USB_SERVICE);
 	if (Build.VERSION.SDK_INT >= 26) {
 	    notificationChannel=new NotificationChannel("gnu.gforth.notifications", "Messages", NotificationManager.IMPORTANCE_DEFAULT);
 	    notificationChannel.enableLights(true);
@@ -677,6 +686,7 @@ public class Gforth
 	    String filedirs[] = {
 		Environment.getExternalStorageDirectory().toString(),
 		getExternalFilesDir(null).toString(),
+		getExternalCacheDir().toString(),
 		getFilesDir().toString() };
 	    for(int i=0; i<filedirs.length; i++) {
 		Log.v(TAG, "filedirs["+i+"]="+filedirs[i]);
@@ -894,6 +904,7 @@ public class Gforth
     public static String[] REQUEST_STRING_NEW = {
 	Manifest.permission.READ_MEDIA_IMAGES,
 	Manifest.permission.READ_MEDIA_VIDEO,
+	Manifest.permission.READ_MEDIA_AUDIO,
     };
     public boolean verifyStoragePermissions(Activity activity) {
 	// Check if we have write permission
@@ -921,6 +932,28 @@ public class Gforth
 	    } else {
 		Log.v(TAG, "Has External Storage Permission");
 	    }
+	} else if(Build.VERSION.SDK_INT >= 33) {
+	    int permission = PackageManager.PERMISSION_GRANTED;
+	    for(int i=0; i < REQUEST_STRING_NEW.length; i++) {
+		int new_perm = checkSelfPermission(REQUEST_STRING_NEW[i]);
+		if (new_perm != PackageManager.PERMISSION_GRANTED) {
+		    permission = new_perm;
+		}
+	    }
+
+	    if (permission != PackageManager.PERMISSION_GRANTED) {
+		// We don't have permission so prompt the user
+		Log.v(TAG, "Request External Storage Permission");
+		for(int i=0; i < REQUEST_STRING_NEW.length; i++) {
+		    if(shouldShowRequestPermissionRationale(REQUEST_STRING_NEW[i])) {
+			Log.v(TAG, "Requires Permission Rationale for " + REQUEST_STRING_NEW[i]);
+		    }
+		}
+		requestPermissions(REQUEST_STRING_NEW, REQUEST_EXTERNAL_STORAGE);
+		return false;
+	    } else {
+		Log.v(TAG, "Has External Media Files Permission");
+	    }
 	}
 	return true;
     }
